%
% Alexandre Kornelius
% October/2017
%
% RBC with labor productivity stochastic growth
%
% Objective:
% - learn how Dynare's automatic detrending works
%

%----------------------------------------------------------------
% 0. Housekeeping
%----------------------------------------------------------------

close all;

%----------------------------------------------------------------
% 1. Variable Definition
%----------------------------------------------------------------

var zz, c, k, y, w, h, R, A, dlogy_obs; % stationary variables
varexo epsilonA, epsilonz;              % exogenous shocks
varobs dlogy_obs;                       % observable variables

%----------------------------------------------------------------
% 2. Calibration
%----------------------------------------------------------------

parameters alpha, beta, varphi, delta, rhoA, sigmaA, rhoz, sigmaz, zztrend;

% Parameters
alpha   = 0.33;     % capital share in production function
beta    = 0.99;     % utility discount factor
varphi  = 0.3585;   % consumption share in utility (target h=0.3333)
delta   = 0.025;    % depreciation rate of capital 

% Shocks
rhoA    = 0.95;     % persistence of production TFP shock
sigmaA  = 0.01;     % std err of production TFP shock
rhoz    = 0.25;     % persistence of growth trend for labor prod. (SAMBA)
sigmaz  = 0.17;     % std err of growth trend for labor productiv. (SAMBA)
zztrend = 0.00731348; % mean stochastic growth trend for labor productivity
                    % growth trend from regression on log of y (0.00731348)

%----------------------------------------------------------------
% 3. Model
%----------------------------------------------------------------

model;

% Households

    % budget constraint
    exp(c) + exp(k) = exp(w)*exp(h) + (R - delta)*exp(k(-1))/exp(zz);

    % FOC h - intra-temporal labor supply decision
    varphi * exp(w) / exp(c) = (1-varphi) / (1-exp(h));

    % FOC k - Euler equation for inter-temporal capital decision
    1 / exp(c) = beta / exp(c(+1)) * (R(+1) - delta) / exp(zz);

% Firms

    % production function
    exp(y) = exp(A) * exp(k(-1))^alpha * exp(h)^(1-alpha) / exp(zz)^alpha;

    % FOC h - intra-temporal labor demand
    exp(w) = (1-alpha) * exp(y) / exp(h);

    % FOC k - intra-temporal capital demand
    R-1 = alpha * exp(y) / exp(k(-1)) * exp(zz);

% Shocks

    % production TFP AR(1) process
    A = rhoA * A(-1) + epsilonA;

    % labor productivity growth rate AR(1) process;
    zz = (1-rhoz)*zztrend + rhoz*zz(-1) + epsilonz;

% Observables
    
    dlogy_obs = y - y(-1) + zz;

end;

%----------------------------------------------------------------
% 4. Computation
%----------------------------------------------------------------

initval;
    zz = zztrend;
    R = exp(zztrend) / beta + delta;
    w = log( (1-alpha) * ((R-1)/alpha)^(alpha/(alpha-1)) );
    h = - log(1 + (1-varphi)/varphi/exp(w) * (exp(w)/(1-alpha) -
        (exp(zztrend)-1+delta)*((R-1)/alpha)^(1/(alpha-1)) ) );
    c = log( exp(h) * (exp(w)/(1-alpha) -
        (exp(zztrend)-1+delta)*((R-1)/alpha)^(1/(alpha-1)) ) );
    k = log( exp(h) * exp(zztrend) * ((R-1)/alpha)^(1/(alpha-1)));
    y = log( exp(k)^alpha * exp(h)^(1-alpha) / exp(zztrend)^alpha);
    A = 0;
    dlogy_obs = zztrend;
end;

steady;
check;

shocks;
    var epsilonA;   stderr sigmaA;
    var epsilonz;   stderr sigmaz;
end;

set_dynare_seed(1234567)
%stoch_simul(order=1, irf=40)
%    y k c 
%    h w R
%    zz A
%;

estimated_params;
    rhoA, beta_pdf, 0.5, 0.20;
    rhoz, beta_pdf, 0.5, 0.20;
    stderr epsilonA, inv_gamma_pdf, 0.01, inf;
    stderr epsilonz, inv_gamma_pdf, 0.01, inf;
end;

estimation(
    datafile='Dados.mat',
    mh_replic=20000,mh_nblocks=2,mh_drop=0.50,mh_jscale=1.15,mode_check
    ,mode_compute=5
%    ,mode_file=RBC_Growth_Detrended_mode
    ,plot_priors=0
%    ,smoother
);

%shock_decomposition y;

%----------------------------------------------------------------
% 5. Results
%----------------------------------------------------------------

save('RBC_Growth_Detrended.mat');
